#!/usr/bin/env bash

# Usage: ./install.sh USER PGDATA


if [ $# -lt 2 ]
then
  echo "Error: lack of params"
  echo "Usage: sudo ./install.sh USER PGDATA [SERVICE_NAME]"
  echo "- USER: database installation user (usually 'omm')"
  echo "- PGDATA: data directory of the database"
  echo "- SERVICE_NAME: (Optional) service name for register to systemd, default: 'mogha'"
  exit 1
else
  USER=$1
  arg2=$2
  PGDATA=${arg2%*/}
  SERVICE="mogha"
  if [ $# -gt 2 ]
  then
    SERVICE=$3
  fi
fi

set -e

CUR_DIR="$(cd "$(dirname "$0")" && pwd)"
source "$CUR_DIR"/common.sh

log "MogHA installation directory: $CUR_DIR"

SERVICE_REGISTRY="/usr/lib/systemd/system/"

GROUP_ID=$(grep -E "^$USER" /etc/passwd | awk -F":" '{print $3}')
GROUP=$(grep -E ":$GROUP_ID" /etc/group | awk -F":" '{print $1}')
log "runtime user: $USER, user group: ${GROUP}"

log "PGDATA=$PGDATA"
ldenv=$(sudo -Hiu "$USER" env | grep 'LD_LIBRARY_PATH')
LD_LIBRARY_PATH=${ldenv#*=}
log "LD_LIBRARY_PATH=$LD_LIBRARY_PATH"
ghenv=$(sudo -Hiu "$USER" env | grep 'GAUSSHOME')
GAUSSHOME=${ghenv#*=}
log "GAUSSHOME=${GAUSSHOME}"
DB_PORT=$(grep -E "^port\s*=" "$PGDATA"/postgresql.conf |  tr -cd "0-9")
if [ ! "$DB_PORT" ]; then
    DB_PORT=5432
fi
log "database port: $DB_PORT"

ARCH=$(uname -m)
log "architecture: $ARCH"
BINARY="mogha"

log "modify owner for installation dir..."
chown -R "$USER":"$GROUP" "$CUR_DIR"


log "generate ${SERVICE}.service file..."
SERVICE_FILE="$CUR_DIR"/"${SERVICE}".service
rm -f "$SERVICE_FILE"
cp "$CUR_DIR"/mogha.service.tmpl "$SERVICE_FILE"
chown "$USER":"$GROUP" "$SERVICE_FILE"
chmod 755 "$SERVICE_FILE"
replaceMacro "$SERVICE_FILE" "__GAUSSHOME__" "$GAUSSHOME"
replaceMacro "$SERVICE_FILE" "__MOGHA_BIN__" "$BINARY"
replaceMacro "$SERVICE_FILE" "__PGDATA__" "$PGDATA"
replaceMacro "$SERVICE_FILE" "__USER__" "$USER"
replaceMacro "$SERVICE_FILE" "__LD_LIBRARY_PATH__" "$LD_LIBRARY_PATH"
replaceMacro "$SERVICE_FILE" "__INSTALLDIR__" "$CUR_DIR"
log "move ${SERVICE}.service to $SERVICE_REGISTRY"
mv "$SERVICE_FILE" $SERVICE_REGISTRY
log "reload systemd"
systemctl daemon-reload
log "${SERVICE} service register successful"


SUDOERFILE="/etc/sudoers.d/$USER"
log "add sudo permissions to $SUDOERFILE for $USER"
echo "$USER ALL=(ALL) NOPASSWD:$(command -v ifconfig)" >> "$SUDOERFILE"
echo "$USER ALL=(ALL) NOPASSWD:$(command -v systemctl)" >> "$SUDOERFILE"


if [ ! -f "${CUR_DIR}/node.conf" ]
then
  log "not found node.conf in current directory, generate it"
  cp "$CUR_DIR"/node.conf.tmpl "$CUR_DIR"/node.conf
  chown "$USER":"${GROUP}" "$CUR_DIR"/node.conf
  chmod 755 "$CUR_DIR"/node.conf
  replaceMacro "$CUR_DIR"/node.conf "__INSTALLDIR__" "$CUR_DIR"
  replaceMacro "$CUR_DIR"/node.conf "__DB_PORT__" "$DB_PORT"
  replaceMacro "$CUR_DIR"/node.conf "__USER__" "$USER"
  replaceMacro "$CUR_DIR"/node.conf "__PGDATA__" "$PGDATA"
  log "node.conf generated, location: ${CUR_DIR}/node.conf"
else
  log "found node.conf already exists, skip"
fi

log "MogHA install successfully!"
echo ""
echo "Please edit ${CUR_DIR}/node.conf first before start MogHA service!!!"
echo ""
echo "Manage MogHA service by systemctl command:"
echo "    Start service:   sudo systemctl start ${SERVICE}"
echo "    Stop service:    sudo systemctl stop ${SERVICE}"
echo "    Restart service: sudo systemctl restart ${SERVICE}"
echo ""
echo "Uninstall MogHA service command:"
echo "    sudo ./uninstall.sh ${SERVICE}"
echo ""

